-- MySQL Script generated by MySQL Workbench
-- Wed Jun 27 19:14:48 2018
-- Model: New Model    Version: 1.0
-- MySQL Workbench Forward Engineering

SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0;
SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0;
SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='TRADITIONAL,ALLOW_INVALID_DATES';

-- -----------------------------------------------------
-- Schema MicroServiceFrameDB
-- -----------------------------------------------------
DROP SCHEMA IF EXISTS `MicroServiceFrameDB` ;

-- -----------------------------------------------------
-- Schema MicroServiceFrameDB
-- -----------------------------------------------------
CREATE SCHEMA IF NOT EXISTS `MicroServiceFrameDB` DEFAULT CHARACTER SET utf8 ;
USE `MicroServiceFrameDB` ;

-- -----------------------------------------------------
-- Table `MicroServiceFrameDB`.`Tenants`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `MicroServiceFrameDB`.`Tenants` ;

CREATE TABLE IF NOT EXISTS `MicroServiceFrameDB`.`Tenants` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `uuid` CHAR(30) NOT NULL,
  `name` VARCHAR(45) NULL,
  `description` VARCHAR(45) NULL,
  `createdAt` TIMESTAMP NULL,
  `modifiedAt` TIMESTAMP NULL,
  PRIMARY KEY (`id`),
  UNIQUE INDEX `uuid_UNIQUE` (`uuid` ASC))
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `MicroServiceFrameDB`.`Directories`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `MicroServiceFrameDB`.`Directories` ;

CREATE TABLE IF NOT EXISTS `MicroServiceFrameDB`.`Directories` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `uuid` CHAR(30) NOT NULL,
  `tenantUUID` CHAR(30) NULL,
  `name` VARCHAR(45) NULL,
  `description` VARCHAR(45) NULL,
  `merchantUUID` VARCHAR(45) NULL,
  `merchantHref` VARCHAR(200) NULL,
  `createdAt` TIMESTAMP NULL,
  `modifiedAt` TIMESTAMP NULL,
  PRIMARY KEY (`id`),
  UNIQUE INDEX `uuid_UNIQUE` (`uuid` ASC),
  INDEX `tenantUUID_idx` (`tenantUUID` ASC))
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `MicroServiceFrameDB`.`Users`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `MicroServiceFrameDB`.`Users` ;

CREATE TABLE IF NOT EXISTS `MicroServiceFrameDB`.`Users` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `uuid` CHAR(30) NOT NULL,
  `directoryUUID` CHAR(30) NULL,
  `name` VARCHAR(45) NULL,
  `email` VARCHAR(45) NULL,
  `mobile` VARCHAR(45) NULL,
  `createdAt` TIMESTAMP NULL,
  `modifiedAt` TIMESTAMP NULL,
  `data` TEXT NULL,
  PRIMARY KEY (`id`),
  UNIQUE INDEX `uuid_UNIQUE` (`uuid` ASC),
  INDEX `directoryUUID_idx` (`directoryUUID` ASC))
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `MicroServiceFrameDB`.`Groups`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `MicroServiceFrameDB`.`Groups` ;

CREATE TABLE IF NOT EXISTS `MicroServiceFrameDB`.`Groups` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `uuid` CHAR(30) NOT NULL,
  `tenantUUID` CHAR(30) NULL,
  `name` VARCHAR(45) NULL,
  `description` VARCHAR(45) NULL,
  `merchantUUID` CHAR(30) NULL,
  `merchantHref` VARCHAR(500) NULL,
  `createdAt` TIMESTAMP NULL,
  `modifiedAt` TIMESTAMP NULL,
  PRIMARY KEY (`id`),
  UNIQUE INDEX `uuid_UNIQUE` (`uuid` ASC),
  INDEX `tenantUUID_idx` (`tenantUUID` ASC))
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `MicroServiceFrameDB`.`UserGroupMemberships`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `MicroServiceFrameDB`.`UserGroupMemberships` ;

CREATE TABLE IF NOT EXISTS `MicroServiceFrameDB`.`UserGroupMemberships` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `uuid` CHAR(30) NOT NULL,
  `userUUID` CHAR(30) NULL,
  `groupUUID` CHAR(30) NULL,
  `createdAt` TIMESTAMP NULL,
  `modifiedAt` TIMESTAMP NULL,
  PRIMARY KEY (`id`),
  UNIQUE INDEX `uuid_UNIQUE` (`uuid` ASC),
  INDEX `userUUID_idx` (`userUUID` ASC),
  INDEX `groupUUID_idx` (`groupUUID` ASC))
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `MicroServiceFrameDB`.`Trees`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `MicroServiceFrameDB`.`Trees` ;

CREATE TABLE IF NOT EXISTS `MicroServiceFrameDB`.`Trees` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `uuid` CHAR(30) NOT NULL,
  `name` VARCHAR(45) NULL,
  `parentTreeUUID` CHAR(30) NULL,
  `createdAt` TIMESTAMP NULL,
  `modifiedAt` TIMESTAMP NULL,
  PRIMARY KEY (`id`),
  UNIQUE INDEX `uuid_UNIQUE` (`uuid` ASC),
  INDEX `upLevelTreeUUID_idx` (`parentTreeUUID` ASC))
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `MicroServiceFrameDB`.`TreeGroupMemberships`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `MicroServiceFrameDB`.`TreeGroupMemberships` ;

CREATE TABLE IF NOT EXISTS `MicroServiceFrameDB`.`TreeGroupMemberships` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `uuid` CHAR(30) NOT NULL,
  `treeUUID` CHAR(30) NULL,
  `groupUUID` CHAR(30) NULL,
  `createdAt` TIMESTAMP NULL,
  `modifiedAt` TIMESTAMP NULL,
  PRIMARY KEY (`id`),
  UNIQUE INDEX `uuid_UNIQUE` (`uuid` ASC),
  INDEX `treeUUID_idx` (`treeUUID` ASC),
  INDEX `groupUUID_idx` (`groupUUID` ASC))
ENGINE = InnoDB;

USE `MicroServiceFrameDB` ;


-- -----------------------------------------------------
-- function getSubTreeUUIDs
-- -----------------------------------------------------

USE `MicroServiceFrameDB`;
DROP function IF EXISTS `MicroServiceFrameDB`.`getSubTreeUUIDs`;

DELIMITER $$
USE `MicroServiceFrameDB`$$
CREATE FUNCTION `getSubTreeUUIDs`(rootUUID char(30)) RETURNS varchar(9999) CHARSET utf8
BEGIN
	DECLARE sTemp VARCHAR(9999);
	DECLARE sTempChd VARCHAR(9999);
    
	SET sTemp = '$';
	SET sTempChd = cast(rootUUID as CHAR(30));
    
	WHILE sTempChd is not null DO
		SELECT GROUP_CONCAT(uuid) INTO sTempChd FROM Trees WHERE FIND_IN_SET(parentTreeUUID, sTempChd) > 0;
        IF sTempChd is not null then 
           SET sTemp = concat(sTemp,',',sTempChd); 
		End if;
	END WHILE;
RETURN sTemp;
END$$

DELIMITER ;

SET SQL_MODE=@OLD_SQL_MODE;
SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS;
SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS;
